@@ -42,7 +42,7 @@
 
 #include <config.h>
 
-/* $Id: fud.c,v 1.22 2001/02/22 19:27:16 ken3 Exp $ */
+/* $Id: fud.c,v 1.22.6.1 2002/12/03 19:26:08 rjs3 Exp $ */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -127,13 +127,12 @@ int begin_handling(void)
 	    }
             for(off = 0; buf[off] != '|' && off < MAXLOGNAME; off++);
             if(off < MAXLOGNAME) {
-		strlcpy(username,buf,off);
+		strncpy(username,buf,off);
             } else {
 		continue;
             }
             q = buf + off + 1;
-            strlcpy(mbox,q,(r - (off + 1)  < MAX_MAILBOX_NAME) ? 
-		    r - (off + 1) : MAX_MAILBOX_NAME);
+            strlcpy(mbox, q, sizeof(mbox));
 
             handle_request(username,mbox,sfrom);
         }
