@@ -40,7 +40,7 @@
  *
  */
 /*
- * $Id: mboxlist.c,v 1.148 2001/06/23 03:14:02 ken3 Exp $
+ * $Id: mboxlist.c,v 1.148.4.1 2002/12/03 19:26:09 rjs3 Exp $
  */
 
 #include <config.h>
@@ -386,7 +386,7 @@ mboxlist_mycreatemailboxcheck(char *name, int mbtype, char *partition,
 	acl[parentacllen] = '\0';
 
 	/* Canonicalize case of parent prefix */
-	strlcpy(name, parent, strlen(parent));
+	strncpy(name, parent, strlen(parent));
     } else { /* parentlen == 0, no parent mailbox */
 	if (!isadmin) {
 	    return IMAP_PERMISSION_DENIED;
