@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.98 2001/07/06 23:26:10 ken3 Exp $
+ * $Id: pop3d.c,v 1.100 2001/08/03 15:33:24 rjs3 Exp $
  */
 #include <config.h>
 
@@ -132,7 +132,7 @@ static mailbox_decideproc_t expungedeleted;
 
 static void cmd_apop(char *user, char *digest);
 static int apop_enabled(void);
-static char popd_apop_chal[300]; /* timestamp (44) + config_servername (256) */
+static char popd_apop_chal[45 + MAXHOSTNAMELEN + 1]; /* <rand.time@hostname> */
 
 static void cmd_auth();
 static void cmd_capa();
@@ -320,6 +320,12 @@ int service_main(int argc, char **argv, char **envp)
        TLS negotiation immediatly */
     if (pop3s == 1) cmd_starttls(1);
 
+    /* Create APOP challenge for banner */
+    if (!sasl_mkchal(popd_saslconn, popd_apop_chal, sizeof(popd_apop_chal), 1)) {
+	syslog(LOG_ERR, "APOP disabled: can't create challenge");
+	*popd_apop_chal = 0;
+    }
+
     prot_printf(popd_out, "+OK %s Cyrus POP3 %s server ready\r\n",
 		apop_enabled() ? popd_apop_chal : config_servername,
 		CYRUS_VERSION);
@@ -794,11 +800,6 @@ static void cmd_starttls(int pop3s)
 	fatal("sasl_setprop() failed: cmd_starttls()", EC_TEMPFAIL);
     }
 
-    /* if authenticated set that */
-    if (external.auth_id != NULL) {
-	popd_userid = external.auth_id;
-    }
-
     /* tell the prot layer about our new layers */
     prot_settls(popd_in, tls_conn);
     prot_settls(popd_out, tls_conn);
@@ -820,18 +821,9 @@ static void cmd_starttls(int pop3s __attribute__((unused)))
 #ifdef HAVE_APOP
 static int apop_enabled(void)
 {
-    static int chal_done = 0;
-
-    /* Create APOP challenge string for banner */
-    if (!chal_done &&
-	!sasl_mkchal(popd_saslconn, popd_apop_chal, sizeof(popd_apop_chal), 1)) {
-	syslog(LOG_WARNING,
-	       "can't create APOP challenge string - APOP disabled");
-	popd_apop_chal[0] = '\0';
-    }
-
-    chal_done = 1;
-    return *popd_apop_chal;
+    /* Check if we have a challenge string */
+    if (!*popd_apop_chal) return 0;
+    return 1;
 }
 
 static void cmd_apop(char *user, char *digest)
