@@ -1,5 +1,5 @@
 /* seen_db.c -- implementation of seen database using per-user berkeley db
-   $Id: seen_db.c,v 1.21 2001/01/05 06:00:03 leg Exp $
+   $Id: seen_db.c,v 1.21.10.1 2002/12/03 19:26:09 rjs3 Exp $
  
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -238,7 +238,7 @@ static int seen_readold(struct seen *seendb,
     while (p < base + offset + linelen && !isspace((int) *p)) p++;
 
     *seenuidsptr = xmalloc(p - buf + 1);
-    strlcpy(*seenuidsptr, buf, p - buf);
+    strlcpy(*seenuidsptr, buf, p - buf + 1);
     (*seenuidsptr)[p - buf] = '\0';
 
     map_free(&base, &len);
