@@ -75,6 +75,7 @@ static int dbinit = 0;
 static DB_ENV *dbenv;
 
 /* other routines call this one when they fail */
+static int commit_txn(struct db *db, struct txn *tid);
 static int abort_txn(struct db *db, struct txn *tid);
 
 static void db_panic(DB_ENV *dbenv, int errno)
@@ -113,12 +114,21 @@ static int init(const char *dbdir, int myflags)
 	dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT, 1);
     }
     dbenv->set_lk_detect(dbenv, CONFIG_DEADLOCK_DETECTION);
+
+    /* XXX should make this value runtime configurable */
     r = dbenv->set_lk_max(dbenv, 50000);
     if (r) {
 	syslog(LOG_ERR, "DBERROR: set_lk_max(): %s", db_strerror(r));
 	abort();
     }
-    /*    dbenv->set_lk_max(dbenv, 10000);*/
+
+    /* XXX should make this value runtime configurable */
+    r = dbenv->set_tx_max(dbenv, 100);
+    if (r) {
+	syslog(LOG_ERR, "DBERROR: set_tx_max(): %s", db_strerror(r));
+	abort();
+    }
+
     dbenv->set_errcall(dbenv, db_err);
     dbenv->set_errpfx(dbenv, "db3");
 
@@ -253,22 +263,25 @@ static int close(struct db *db)
     return r;
 }
 
-static int gettid(struct txn **mytid, DB_TXN **tid)
+static int gettid(struct txn **mytid, DB_TXN **tid, char *where)
 {
     int r;
 
     if (mytid) {
 	if (*mytid) {
+  	    assert((txn_id((DB_TXN *)*mytid) != 0));
 	    *tid = (DB_TXN *) *mytid;
+	    if (CONFIG_DB_VERBOSE)
+		syslog(LOG_DEBUG, "%s: reusing txn %lu", where, txn_id(*tid));
 	} else {
 	    r = txn_begin(dbenv, NULL, tid, 0);
 	    if (r != 0) {
-		syslog(LOG_ERR, "DBERROR: error beginning txn: %s", 
+		syslog(LOG_ERR, "DBERROR: error beginning txn (%s): %s", where,
 		       db_strerror(r));
 		return CYRUSDB_IOERROR;
 	    }
 	    if (CONFIG_DB_VERBOSE)
-		syslog(LOG_DEBUG, "starting txn %lu", txn_id(*tid));
+		syslog(LOG_DEBUG, "%s: starting txn %lu", where, txn_id(*tid));
 	}
 	*mytid = (struct txn *) *tid;
     }
@@ -288,7 +301,7 @@ static int myfetch(struct db *mydb,
 	
     assert(dbinit && db);
 
-    r = gettid(mytid, &tid);
+    r = gettid(mytid, &tid, "myfetch");
     if (r) return r;
 
     memset(&k, 0, sizeof(k));
@@ -297,7 +310,7 @@ static int myfetch(struct db *mydb,
     k.data = (char *) key;
     k.size = keylen;
 
-    r = db->get(db, tid, &k, &d, 0);
+    r = db->get(db, tid, &k, &d, flags);
     switch (r) {
     case 0:
 	if (data) *data = d.data;
@@ -309,7 +322,10 @@ static int myfetch(struct db *mydb,
 	r = 0;
 	break;
     case DB_LOCK_DEADLOCK:
-	if (mytid) abort_txn(mydb, *mytid);
+	if (mytid) {
+	    abort_txn(mydb, *mytid);
+	    *mytid = NULL;
+	}
 	r = CYRUSDB_AGAIN;
 	break;
     default:
@@ -382,7 +398,7 @@ static int foreach(struct db *mydb,
     /* k.flags |= DB_DBT_REALLOC;
        d.flags |= DB_DBT_REALLOC;*/
 
-    r = gettid(mytid, &tid);
+    r = gettid(mytid, &tid, "foreach");
     if (r) return r;
 
     if (0) {
@@ -481,12 +497,14 @@ static int foreach(struct db *mydb,
     case DB_LOCK_DEADLOCK:	/* erg, we're in a txn! */
 	if (mytid) {
 	    abort_txn(mydb, *mytid);
+	    *mytid = NULL;
 	}
 	r = CYRUSDB_AGAIN;
 	break;
     default:
 	if (mytid) {
-	    abort_txn(mydb, *mytid);
+	    abort_txn(mydb, *mytid); 
+	    *mytid = NULL;
 	}
 	syslog(LOG_ERR, "DBERROR: error advancing: %s",  db_strerror(r));
 	r = CYRUSDB_IOERROR;
@@ -512,7 +530,7 @@ static int mystore(struct db *mydb,
     assert(dbinit && db);
     assert(key && keylen);
 
-    r = gettid(mytid, &tid);
+    r = gettid(mytid, &tid, "mystore");
     if (r) return r;
 
     memset(&k, 0, sizeof(k));
@@ -528,12 +546,12 @@ static int mystore(struct db *mydb,
     restart:
 	r = txn_begin(dbenv, NULL, &tid, 0);
 	if (r != 0) {
-	    syslog(LOG_ERR, "DBERROR: error beginning txn: %s", 
+	    syslog(LOG_ERR, "DBERROR: mystore: error beginning txn: %s", 
 		   db_strerror(r));
 	    return CYRUSDB_IOERROR;
 	}
 	if (CONFIG_DB_VERBOSE)
-	    syslog(LOG_DEBUG, "starting txn %lu", txn_id(tid));
+	    syslog(LOG_DEBUG, "mystore: starting txn %lu", txn_id(tid));
     }
     r = db->put(db, tid, &k, &d, 0);
     if (!mytid) {
@@ -542,10 +560,10 @@ static int mystore(struct db *mydb,
 	    int r2;
 
 	    if (CONFIG_DB_VERBOSE)
-		syslog(LOG_DEBUG, "aborting txn %lu", txn_id(tid));
+		syslog(LOG_DEBUG, "mystore: aborting txn %lu", txn_id(tid));
 	    r2 = txn_abort(tid);
 	    if (r2) {
-		syslog(LOG_ERR, "DBERROR: error aborting txn: %s", 
+		syslog(LOG_ERR, "DBERROR: mystore: error aborting txn: %s", 
 		       db_strerror(r));
 		return CYRUSDB_IOERROR;
 	    }
@@ -555,17 +573,20 @@ static int mystore(struct db *mydb,
 	    }
 	} else {
 	    if (CONFIG_DB_VERBOSE)
-		syslog(LOG_DEBUG, "committing txn %lu", txn_id(tid));
+		syslog(LOG_DEBUG, "mystore: committing txn %lu", txn_id(tid));
 	    r = txn_commit(tid, 0);
 	}
     }
 
-    if (r != 0) {
-	if (mytid) abort_txn(mydb, *mytid);
+    if ( r != 0) {
+	if (mytid) {
+	    abort_txn(mydb, *mytid);
+	    *mytid = NULL;
+	}
 	if (r == DB_LOCK_DEADLOCK) {
 	    r = CYRUSDB_AGAIN;
 	} else {
-	    syslog(LOG_ERR, "DBERROR: error storing %s: %s",
+	    syslog(LOG_ERR, "DBERROR: mystore: error storing %s: %s",
 		   key, db_strerror(r));
 	    r = CYRUSDB_IOERROR;
 	}
@@ -602,7 +623,7 @@ static int delete(struct db *mydb,
     assert(dbinit && db);
     assert(key && keylen);
 
-    r = gettid(mytid, &tid);
+    r = gettid(mytid, &tid, "delete");
     if (r) return r;
 
     memset(&k, 0, sizeof(k));
@@ -615,12 +636,12 @@ static int delete(struct db *mydb,
 	/* start txn for the write */
 	r = txn_begin(dbenv, NULL, &tid, 0);
 	if (r != 0) {
-	    syslog(LOG_ERR, "DBERROR: error beginning txn: %s", 
+	    syslog(LOG_ERR, "DBERROR: delete: error beginning txn: %s", 
 		   db_strerror(r));
 	    return CYRUSDB_IOERROR;
 	}
 	if (CONFIG_DB_VERBOSE)
-	    syslog(LOG_DEBUG, "starting txn %lu", txn_id(tid));
+	    syslog(LOG_DEBUG, "delete: starting txn %lu", txn_id(tid));
     }
     r = db->del(db, tid, &k, 0);
     if (!mytid) {
@@ -628,10 +649,10 @@ static int delete(struct db *mydb,
 	if (r) {
 	    int r2;
 	    if (CONFIG_DB_VERBOSE)
-		syslog(LOG_DEBUG, "aborting txn %lu", txn_id(tid));
+		syslog(LOG_DEBUG, "delete: aborting txn %lu", txn_id(tid));
 	    r2 = txn_abort(tid);
 	    if (r2) {
-		syslog(LOG_ERR, "DBERROR: error aborting txn: %s", 
+		syslog(LOG_ERR, "DBERROR: delete: error aborting txn: %s", 
 		       db_strerror(r));
 		return CYRUSDB_IOERROR;
 	    }
@@ -641,17 +662,20 @@ static int delete(struct db *mydb,
 	    }
 	} else {
 	    if (CONFIG_DB_VERBOSE)
-		syslog(LOG_DEBUG, "committing txn %lu", txn_id(tid));
+		syslog(LOG_DEBUG, "delete: committing txn %lu", txn_id(tid));
 	    r = txn_commit(tid, 0);
 	}
     }
 
     if (r != 0) {
-	if (mytid) abort_txn(mydb, *mytid);
+	if (mytid) {
+	    abort_txn(mydb, *mytid);
+	    *mytid = NULL;
+	}
 	if (r == DB_LOCK_DEADLOCK) {
 	    r = CYRUSDB_AGAIN;
 	} else {
-	    syslog(LOG_ERR, "DBERROR: error deleting %s: %s",
+	    syslog(LOG_ERR, "DBERROR: delete: error deleting %s: %s",
 		   key, db_strerror(r));
 	    r = CYRUSDB_IOERROR;
 	}
@@ -668,17 +692,17 @@ static int commit_txn(struct db *db, struct txn *tid)
     assert(dbinit && tid);
 
     if (CONFIG_DB_VERBOSE)
-	syslog(LOG_DEBUG, "committing txn %lu", txn_id(t));
+	syslog(LOG_DEBUG, "commit_txn: committing txn %lu", txn_id(t));
     r = txn_commit(t, 0);
     switch (r) {
     case 0:
 	break;
     case EINVAL:
-	syslog(LOG_WARNING, "tried to commit an already aborted transaction");
+	syslog(LOG_WARNING, "commit_txn: tried to commit an already aborted transaction");
 	r = CYRUSDB_IOERROR;
 	break;
     default:
-	syslog(LOG_ERR, "DBERROR: failed on commit: %s",
+	syslog(LOG_ERR, "DBERROR: commit_txn  failed on commit: %s",
 	       db_strerror(r));
 	r = CYRUSDB_IOERROR;
 	break;
@@ -695,10 +719,10 @@ static int abort_txn(struct db *db, struct txn *tid)
     assert(dbinit && tid);
 
     if (CONFIG_DB_VERBOSE)
-	syslog(LOG_DEBUG, "aborting txn %lu", txn_id(t));
+	syslog(LOG_DEBUG, "abort_txn: aborting txn %lu", txn_id(t));
     r = txn_abort(t);
     if (r != 0) {
-	syslog(LOG_ERR, "DBERROR: error aborting txn: %s", db_strerror(r));
+	syslog(LOG_ERR, "DBERROR: abort_txn: error aborting txn: %s", db_strerror(r));
 	return CYRUSDB_IOERROR;
     }
 
