@@ -1,6 +1,6 @@
 /* script.c -- sieve script functions
  * Larry Greenfield
- * $Id: script.c,v 1.40 2000/12/18 04:53:43 leg Exp $
+ * $Id: script.c,v 1.42.2.1 2002/12/03 20:30:46 rjs3 Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -511,14 +511,16 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 			    strcpy(buf, "Automated reply");
 			} else {
 			    /* s[0] contains the original subject */
-			    while (!strncasecmp(s[0], "Re: ", 4)) {
-				s[0] += 4;
+			    const char *origsubj = s[0];
+			    
+			    while (!strncasecmp(origsubj, "Re: ", 4)) {
+				origsubj += 4;
 			    }
-			    snprintf(buf, sizeof(buf), "Re: %s", s[0]);
+			    snprintf(buf, sizeof(buf), "Re: %s", origsubj);
 			}
 		    } else {
 			/* user specified subject */
-			strncpy(buf, c->u.v.subject, sizeof(buf));
+			strlcpy(buf, c->u.v.subject, sizeof(buf));
 		    }
 
 		    /* who do we want the message coming from? */
@@ -528,14 +530,20 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 			fromaddr = myaddr;
 		    }
 		
-		    res = do_vacation(actions, reply_to, strdup(fromaddr),
-				      strdup(buf),
-				      c->u.v.message, c->u.v.days, c->u.v.mime);
-		
-		     if (res == SIEVE_RUN_ERROR)
-			 *errmsg = "Vacation can not be used with Reject or Vacation";
+		    res = do_vacation(actions, reply_to, xstrdup(fromaddr),
+				      xstrdup(buf), c->u.v.message,
+				      c->u.v.days, c->u.v.mime);
+
+		    if (res && reply_to) free(reply_to);
+		    
+		    if (res == SIEVE_RUN_ERROR) {
+			*errmsg =
+			    "Vacation can not be used with Reject or Vacation";
+		    }
 
 		} else {
+		    if (reply_to) free(reply_to);
+		    
 		    if (l != SIEVE_DONE) res = -1; /* something went wrong */
 		}
 		if (myaddr) free(myaddr);
@@ -675,7 +683,7 @@ static int sieve_addflag(sieve_imapflags_t *imapflags, char *flag)
 	imapflags->flag =
 	    (char **) xrealloc((char *)imapflags->flag,
 			       imapflags->nflags*sizeof(char *));
-	imapflags->flag[imapflags->nflags-1] = strdup(flag);
+	imapflags->flag[imapflags->nflags-1] = xstrdup(flag);
     }
  
     return SIEVE_OK;
@@ -1024,9 +1032,10 @@ int sieve_execute_script(sieve_script_t *s, void *message_context)
     if ((ret != SIEVE_OK) && s->interp.err) {
 	char buf[1024];
 	if (lastaction == -1) /* we never executed an action */
-	    sprintf(buf, "%s", errmsg ? errmsg : sieve_errstr(ret));
+	    snprintf(buf, sizeof(buf), "%s",
+		     errmsg ? errmsg : sieve_errstr(ret));
 	else
-	    sprintf(buf, "%s: %s", action_to_string(lastaction),
+	    snprintf(buf, sizeof(buf), "%s: %s", action_to_string(lastaction),
 		    errmsg ? errmsg : sieve_errstr(ret));
  
 	ret |= s->interp.execute_err(buf, s->interp.interp_context,
