@@ -1,5 +1,5 @@
 /* sieve_interface.h -- interface for deliver
- * $Id: sieve_interface.h,v 1.13 2000/05/28 22:45:59 leg Exp $
+ * $Id: sieve_interface.h,v 1.14 2000/08/04 18:40:31 leg Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -126,11 +126,12 @@ int sieve_register_size(sieve_interp_t *interp, sieve_get_size *f);
 int sieve_register_header(sieve_interp_t *interp, sieve_get_header *f);
 int sieve_register_envelope(sieve_interp_t *interp, sieve_get_envelope *f);
 
-typedef int sieve_parse_error(int lineno, char *msg, void *interp_context,
+typedef int sieve_parse_error(int lineno, const char *msg, 
+			      void *interp_context,
 			      void *script_context);
 int sieve_register_parse_error(sieve_interp_t *interp, sieve_parse_error *f);
 
-typedef int sieve_execute_error(char *msg, void *interp_context,
+typedef int sieve_execute_error(const char *msg, void *interp_context,
 				void *script_context, void *message_context);
 int sieve_register_execute_error(sieve_interp_t *interp, 
 				 sieve_execute_error *f);
